<?php
/**
 * Plugin Name: Add Links to Images in Single Posts
 * Plugin URI: https://mbsm.tn
 * Description: Automatically wraps all images in single posts with a link pointing to the image URL, using the post title for the alt and title attributes. This helps improve SEO and accessibility by ensuring images are linked and properly described.
 * Version: 1.6
 * Author: Mbsmgroup
 * Author URI: https://mbsm.pro
 * License: GPL2
 * Text Domain: add-image-links
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit; // Prevent direct access
}

// Add plugin settings page
function add_image_links_settings_page() {
    add_options_page(
        'Add Image Links Settings by MBSMGROUP', // Page Title
        'Add Image Links', // Menu Title
        'manage_options', // Capability required to view the page
        'add_image_links', // Menu Slug
        'add_image_links_settings_page_html' // Callback function to display the page
    );
}
add_action('admin_menu', 'add_image_links_settings_page');

// Callback function to display the settings page
function add_image_links_settings_page_html() {
    if (!current_user_can('manage_options')) {
        return;
    }

    ?>
    <div class="wrap">
        <h1><?php esc_html_e('Add Image Links Settings by MBSMGROUP', 'add-image-links'); ?></h1>

        <h2><?php esc_html_e('Installation Instructions', 'add-image-links'); ?></h2>
        <p><?php esc_html_e('1. Download the plugin ZIP file.', 'add-image-links'); ?></p>
        <p><?php esc_html_e('2. Go to the WordPress Dashboard > Plugins > Add New > Upload Plugin.', 'add-image-links'); ?></p>
        <p><?php esc_html_e('3. Choose the ZIP file and click "Install Now".', 'add-image-links'); ?></p>
        <p><?php esc_html_e('4. Once installed, click "Activate".', 'add-image-links'); ?></p>

        <h2><?php esc_html_e('Tips', 'add-image-links'); ?></h2>
        <ul>
            <li><?php esc_html_e('Enable the feature to automatically wrap images with links to improve SEO and user experience.', 'add-image-links'); ?></li>
            <li><?php esc_html_e('Make sure your images have descriptive alt text for better accessibility.', 'add-image-links'); ?></li>
            <li><?php esc_html_e('You can customize the default alt and title text for images in the settings.', 'add-image-links'); ?></li>
        </ul>

        <h2><?php esc_html_e('About the Developer', 'add-image-links'); ?></h2>
        <p><?php esc_html_e('This plugin is developed by MBSMGROUP.', 'add-image-links'); ?></p>
        <p><?php esc_html_e('For support or questions, please visit: ', 'add-image-links'); ?>
            <a href="https://mbsm.pro" target="_blank"><?php esc_html_e('MBSMGROUP Website', 'add-image-links'); ?></a>
        </p>

        <form action="options.php" method="post">
            <?php
            settings_fields('add_image_links_options_group'); // Security field
            do_settings_sections('add_image_links'); // Output all settings fields
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Register plugin settings
function add_image_links_register_settings() {
    // Register settings
    register_setting('add_image_links_options_group', 'add_image_links_options');

    // Add a section
    add_settings_section(
        'add_image_links_section', // Section ID
        __('General Settings', 'add-image-links'), // Title
        null, // Callback function for description
        'add_image_links' // Page
    );

    // Add settings fields
    add_settings_field(
        'enable_image_links', // Field ID
        __('Enable Image Links', 'add-image-links'), // Field Title
        'add_image_links_enable_field_html', // Callback to display field
        'add_image_links', // Page
        'add_image_links_section' // Section
    );

    add_settings_field(
        'default_alt_text', 
        __('Default Alt Text', 'add-image-links'), 
        'add_image_links_alt_field_html', 
        'add_image_links', 
        'add_image_links_section'
    );

    add_settings_field(
        'default_title_text', 
        __('Default Title Text', 'add-image-links'), 
        'add_image_links_title_field_html', 
        'add_image_links', 
        'add_image_links_section'
    );
}
add_action('admin_init', 'add_image_links_register_settings');

// Settings Field HTML Callbacks
function add_image_links_enable_field_html() {
    $options = get_option('add_image_links_options');
    ?>
    <input type="checkbox" name="add_image_links_options[enable_image_links]" value="1" <?php checked(1, isset($options['enable_image_links']) ? $options['enable_image_links'] : 0); ?> />
    <label for="enable_image_links"><?php esc_html_e('Enable wrapping images in links', 'add-image-links'); ?></label>
    <?php
}

function add_image_links_alt_field_html() {
    $options = get_option('add_image_links_options');
    ?>
    <input type="text" name="add_image_links_options[default_alt_text]" value="<?php echo esc_attr(isset($options['default_alt_text']) ? $options['default_alt_text'] : ''); ?>" />
    <p class="description"><?php esc_html_e('Default alt text used for images if no alt attribute is defined.', 'add-image-links'); ?></p>
    <?php
}

function add_image_links_title_field_html() {
    $options = get_option('add_image_links_options');
    ?>
    <input type="text" name="add_image_links_options[default_title_text]" value="<?php echo esc_attr(isset($options['default_title_text']) ? $options['default_title_text'] : ''); ?>" />
    <p class="description"><?php esc_html_e('Default title text used for images if no title attribute is defined.', 'add-image-links'); ?></p>
    <?php
}

// Modify the content based on plugin settings
function add_external_links_to_images_with_post_title($content) {
    if (is_single()) { // Only apply to single post pages
        $options = get_option('add_image_links_options');
        
        if (isset($options['enable_image_links']) && $options['enable_image_links'] == 1) {
            global $post; // Get current post object
            $post_title = isset($post->post_title) ? $post->post_title : ''; // Get post title

            // Default texts from settings
            $default_alt_text = isset($options['default_alt_text']) ? $options['default_alt_text'] : __('Image', 'add-image-links');
            $default_title_text = isset($options['default_title_text']) ? $options['default_title_text'] : __('Click to view image', 'add-image-links');

            // Use a callback to process each image in the post content
            $content = preg_replace_callback(
                '/<img([^>]+)src="([^"]+)"([^>]*)>/i',
                function ($matches) use ($post_title, $default_alt_text, $default_title_text) {
                    $image_url = $matches[2]; // Image URL
                    $image_html = $matches[0]; // Full image tag

                    // Retrieve existing alt and title attributes if they exist
                    preg_match('/alt="([^"]*)"/i', $image_html, $alt_matches);
                    preg_match('/title="([^"]*)"/i', $image_html, $title_matches);

                    // Default alt to post title if not set, and make it more descriptive
                    $alt_text = !empty($alt_matches[1]) ? $alt_matches[1] : $default_alt_text . ' - ' . $post_title;
                    // Default title to post title if not set, and make it more descriptive
                    $title_text = !empty($title_matches[1]) ? $title_matches[1] : $default_title_text . ' - ' . $post_title;

                    // Update the image HTML by adding alt and title attributes if necessary
                    $updated_image_html = preg_replace(
                        '/<img/i',
                        '<img alt="' . esc_attr($alt_text) . '" title="' . esc_attr($title_text) . '"',
                        $image_html
                    );

                    // Wrap image in an external link
                    return '<a href="' . esc_url($image_url) . '" target="_blank" rel="noopener noreferrer" title="' . esc_attr($title_text) . '">' . $updated_image_html . '</a>';
                },
                $content
            );
        }
    }
    return $content;
}

add_filter('the_content', 'add_external_links_to_images_with_post_title');
?>
