<?php
/**
 * Plugin Name: Image Featured Replacer
 * Description: A plugin to track stages and select a featured image from within the post if it is missing, broken, or empty. If no image is available, a default image is set.
 * Version: 1.3
 * Author: Mbsmgroup
 * License: GPL2
 */

// Add a custom menu page to the WordPress admin
function custom_article_control_menu() {
    add_menu_page(
        'Posts without Featured Image', // Page title
        'Posts without Featured Image', // Menu title
        'manage_options',               // Capability
        'no-featured-image-posts',      // Menu slug
        'custom_no_featured_image_page', // Function to display the page
        'dashicons-format-image',       // Icon
        20                              // Position in menu
    );
}
add_action('admin_menu', 'custom_article_control_menu');

// Display the page for posts without featured images
function custom_no_featured_image_page() {
    ?>
    <div class="wrap">
        <h1>Posts Without Featured Image</h1>
        
        <form method="post" action="">
            <input type="submit" name="search_no_featured_image" value="Show Posts" class="button button-primary" />
        </form>

        <?php
        if (isset($_POST['search_no_featured_image'])) {
            custom_search_no_featured_image_posts();
        }
        ?>
    </div>
    <?php
}

// Search for posts that do not have a featured image
function custom_search_no_featured_image_posts() {
    // Search for posts without a featured image
    $args = array(
        'post_type'   => 'post',
        'post_status' => 'any',  // All statuses (drafts, published)
        'posts_per_page' => 10,   // Display 10 posts
        'meta_query' => array(
            array(
                'key'     => '_thumbnail_id',
                'compare' => 'NOT EXISTS', // Search for posts without a featured image
            ),
        ),
    );

    $query = new WP_Query($args);

    if ($query->have_posts()) {
        echo '<h2>Posts Without Featured Images:</h2>';
        echo '<ul>';
        while ($query->have_posts()) {
            $query->the_post();
            echo '<li>';
            echo '<a href="' . get_edit_post_link() . '" target="_blank">' . get_the_title() . '</a>';
            echo '</li>';
            // Add the "Set Featured Image" link in a separate line
            echo '<p><a href="' . add_query_arg('edit_article', get_the_ID(), menu_page_url('no-featured-image-posts', false)) . '" target="_blank">Set Featured Image</a></p>';
        }
        echo '</ul>';
        wp_reset_postdata();
    } else {
        echo '<p>No posts without featured images found.</p>';
    }
}

// Edit post (set a featured image from attached images or input an image URL)
function custom_edit_article($post_id) {
    $post = get_post($post_id);

    // Get attached images
    $attachments = get_attached_media('image', $post_id);
    ?>
    <h2>Set Featured Image for Post: <?php echo esc_html($post->post_title); ?></h2>

    <?php if ($attachments) : ?>
        <form method="post" action="">
            <p><strong>Choose a featured image from attached images:</strong></p>
            <ul style="list-style-type: none;">
                <?php foreach ($attachments as $attachment) : ?>
                    <li>
                        <label>
                            <input type="radio" name="featured_image" value="<?php echo $attachment->ID; ?>" />
                            <img src="<?php echo wp_get_attachment_url($attachment->ID); ?>" alt="<?php echo esc_attr($attachment->post_title); ?>" style="max-width: 150px; margin-right: 10px;" />
                            <?php echo esc_html($attachment->post_title); ?>
                        </label>
                    </li>
                <?php endforeach; ?>
            </ul>
            <p><input type="submit" name="update_thumbnail" value="Set Featured Image" class="button button-primary" /></p>
        </form>
    <?php else : ?>
        <p>No attached images for this post.</p>
        <form method="post" action="">
            <p><strong>Enter an image URL to set it as the featured image:</strong></p>
            <input type="text" name="featured_image_url" value="" placeholder="Enter image URL" style="width: 100%;" />
            <p><input type="submit" name="update_thumbnail_url" value="Set Featured Image" class="button button-primary" /></p>
        </form>
    <?php endif; ?>

    <?php
    // Handle setting the featured image from attached images
    if (isset($_POST['update_thumbnail']) && isset($_POST['featured_image'])) {
        $selected_image_id = intval($_POST['featured_image']);
        custom_update_featured_image($post_id, $selected_image_id);
    }

    // Handle setting the featured image from URL input
    if (isset($_POST['update_thumbnail_url']) && !empty($_POST['featured_image_url'])) {
        $image_url = esc_url($_POST['featured_image_url']);
        custom_update_featured_image_from_url($post_id, $image_url);
    }
}

// Update the featured image with an attached image
function custom_update_featured_image($post_id, $image_id) {
    // Set the featured image for the post
    if (has_post_thumbnail($post_id)) {
        delete_post_thumbnail($post_id); // Remove previous featured image
    }
    set_post_thumbnail($post_id, $image_id);
    echo '<div class="updated"><p>Featured image set successfully.</p></div>';
}

// Update the featured image from an image URL
function custom_update_featured_image_from_url($post_id, $image_url) {
    // Download the image from URL and set it as featured image
    $image_id = media_sideload_image($image_url, $post_id, null, 'id');
    if (!is_wp_error($image_id)) {
        set_post_thumbnail($post_id, $image_id);
        echo '<div class="updated"><p>Featured image set successfully.</p></div>';
    } else {
        echo '<div class="error"><p>Error occurred while downloading the image from the URL.</p></div>';
    }
}

// Display the edit post form if it was selected
function custom_edit_article_page() {
    if (isset($_GET['edit_article'])) {
        $post_id = $_GET['edit_article'];
        custom_edit_article($post_id);
    }
}
add_action('admin_notices', 'custom_edit_article_page');
