<?php
/*
Plugin Name: Mbsm Image Sitemap Generator
Plugin URI: https://mbsm.tn/
Description: Generates an XML sitemap for the latest 2000 images on the website. Deletes old files before generating new ones.
Version: 1.0
Author: Mbsmgroup
Author URI: https://mbsmgroup.tn
License: GPLv3
*/

class Mbsm_Image_Sitemap_Generator {

    private string $sitemap_folder = 'mbsm-sitemap'; // ??? ?????? ??????
    private string $file_name = 'mbsm'; // ??? ????? ?????????

    public function __construct() {
        add_action('admin_menu', [$this, 'add_admin_menu']);
    }

    /**
     * Adds an admin menu page.
     */
    public function add_admin_menu() {
        add_menu_page(
            'MBSM Image Sitemap',  // Page title
            'MBSM Image Sitemap',  // Menu title
            'manage_options', // Capability
            'mbsm-image-sitemap',   // Menu slug
            [$this, 'render_admin_page'], // Callback function
            'dashicons-format-gallery', // Icon
            20  // Position
        );
    }

    /**
     * Renders the admin page.
     */
    public function render_admin_page() {
        // Check if the form is submitted
        if (isset($_POST['generate_sitemap'])) {
            $this->generate_sitemap();
            echo '<div class="notice notice-success"><p>Image sitemap and files generated successfully!</p></div>';
        }

        $upload_dir = wp_upload_dir();
        $sitemap_folder_path = $upload_dir['basedir'] . '/' . $this->sitemap_folder . '/';

        $sitemap_url = $upload_dir['baseurl'] . '/' . $this->sitemap_folder . '/' . $this->file_name . '.xml';
        $sitemap_path = $sitemap_folder_path . $this->file_name . '.xml';

        $files = [
            'xml' => $sitemap_path,
            'txt' => $sitemap_folder_path . $this->file_name . '.txt',
            'html' => $sitemap_folder_path . $this->file_name . '.html',
            'zip' => $sitemap_folder_path . $this->file_name . '.zip',
            'php' => $sitemap_folder_path . $this->file_name . '.php',
        ];
        ?>
        <div class="wrap">
            <h1>MBSM Image Sitemap Generator</h1>
            <p>Click the button below to generate the sitemap for the latest 2000 images on the website.</p>
            <form method="post" action="">
                <input type="hidden" name="generate_sitemap" value="1">
                <?php submit_button('Generate Image Sitemap'); ?>
            </form>

            <?php if (file_exists($sitemap_path)) : ?>
                <h2>Generated Files</h2>
                <table class="wp-list-table widefat fixed striped">
                    <thead>
                        <tr>
                            <th>File Name</th>
                            <th>File URL</th>
                            <th>File Size</th>
                            <th>Last Modified</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php foreach ($files as $type => $path) : ?>
                            <?php if (file_exists($path)) : ?>
                                <tr>
                                    <td><?php echo esc_html(basename($path)); ?></td>
                                    <td>
                                        <a href="<?php echo esc_url($upload_dir['baseurl'] . '/' . $this->sitemap_folder . '/' . basename($path)); ?>" target="_blank">
                                            <?php echo esc_html(basename($path)); ?>
                                        </a>
                                    </td>
                                    <td><?php echo esc_html(size_format(filesize($path))); ?></td>
                                    <td><?php echo esc_html(date('F d, Y H:i:s', filemtime($path))); ?></td>
                                </tr>
                            <?php endif; ?>
                        <?php endforeach; ?>
                    </tbody>
                </table>
            <?php else : ?>
                <p>No sitemap found. Please generate the sitemap first.</p>
            <?php endif; ?>
        </div>
        <?php
    }

    /**
     * Scans the website for the latest 2000 images and generates the sitemap and additional files.
     */
    public function generate_sitemap() {
        $upload_dir = wp_upload_dir();
        $sitemap_folder_path = $upload_dir['basedir'] . '/' . $this->sitemap_folder . '/';

        // ????? ?????? ??? ?? ??? ???????
        if (!file_exists($sitemap_folder_path)) {
            wp_mkdir_p($sitemap_folder_path);
        }

        // ??? ???? ??????? ??????? ?? ??????
        $this->delete_old_files($sitemap_folder_path);

        $images = $this->scan_website_for_images();

        // Generate XML sitemap
        $xml_content = $this->generate_xml_content($images);
        $this->write_file($sitemap_folder_path . $this->file_name . '.xml', $xml_content);

        // Generate additional files
        $this->generate_txt_file($sitemap_folder_path . $this->file_name . '.txt', $images);
        $this->generate_html_file($sitemap_folder_path . $this->file_name . '.html', $images);
        $this->generate_zip_file($sitemap_folder_path . $this->file_name . '.zip', $images);
        $this->generate_php_file($sitemap_folder_path . $this->file_name . '.php', $images);
    }

    /**
     * Deletes all files in the specified folder.
     *
     * @param string $folder_path The path to the folder.
     */
    private function delete_old_files(string $folder_path): void {
        if (is_dir($folder_path)) {
            $files = glob($folder_path . '*'); // ??? ???? ??????? ?? ??????
            foreach ($files as $file) {
                if (is_file($file)) {
                    unlink($file); // ??? ?????
                }
            }
        }
    }

    /**
     * Scans the website for the latest 2000 images.
     *
     * @return array The list of images with their URLs and modification times.
     */
    private function scan_website_for_images(): array {
        $images = [];
        $attachments = get_posts([
            'post_type' => 'attachment',
            'post_mime_type' => 'image',
            'numberposts' => 2000, // ??? ??? 2000 ????
            'post_status' => 'inherit',
            'orderby' => 'date', // ????? ??? ???????
            'order' => 'DESC', // ?? ?????? ??? ??????
        ]);

        foreach ($attachments as $attachment) {
            $image_url = wp_get_attachment_url($attachment->ID);
            $image_path = get_attached_file($attachment->ID);
            $mod_time = gmdate('c', filemtime($image_path));

            $images[] = [
                'url' => $image_url,
                'mod' => $mod_time,
            ];
        }

        return $images;
    }

    /**
     * Generates XML content for the sitemap.
     *
     * @param array $images The list of images to include in the sitemap.
     * @return string The XML content.
     */
    private function generate_xml_content(array $images): string {
        $xml = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL;
        $xml .= '<urlset xmlns="https://www.sitemaps.org/schemas/sitemap/0.9">' . PHP_EOL;

        foreach ($images as $image) {
            $xml .= '<url>' . PHP_EOL;
            $xml .= '<loc>' . esc_url($image['url']) . '</loc>' . PHP_EOL;
            $xml .= '<lastmod>' . esc_html($image['mod']) . '</lastmod>' . PHP_EOL;
            $xml .= '</url>' . PHP_EOL;
        }

        $xml .= '</urlset>';
        return $xml;
    }

    /**
     * Generates a TXT file.
     *
     * @param string $file_path The path where the TXT file will be saved.
     * @param array $images The list of images to include in the TXT file.
     */
    private function generate_txt_file(string $file_path, array $images): void {
        $txt_content = '';
        foreach ($images as $image) {
            $txt_content .= $image['url'] . PHP_EOL;
        }
        $this->write_file($file_path, $txt_content);
    }

    /**
     * Generates an HTML file.
     *
     * @param string $file_path The path where the HTML file will be saved.
     * @param array $images The list of images to include in the HTML file.
     */
    private function generate_html_file(string $file_path, array $images): void {
        $html_content = '<!DOCTYPE html><html><head><title>Image Sitemap</title></head><body><ul>';
        foreach ($images as $image) {
            $html_content .= '<li><a href="' . esc_url($image['url']) . '">' . esc_html($image['url']) . '</a></li>';
        }
        $html_content .= '</ul></body></html>';
        $this->write_file($file_path, $html_content);
    }

    /**
     * Generates a ZIP file.
     *
     * @param string $file_path The path where the ZIP file will be saved.
     * @param array $images The list of images to include in the ZIP file.
     */
    private function generate_zip_file(string $file_path, array $images): void {
        $zip = new ZipArchive();
        if ($zip->open($file_path, ZipArchive::CREATE) === TRUE) {
            foreach ($images as $image) {
                $image_path = str_replace(wp_upload_dir()['baseurl'], wp_upload_dir()['basedir'], $image['url']);
                $zip->addFile($image_path, basename($image_path));
            }
            $zip->close();
        }
    }

    /**
     * Generates a PHP file.
     *
     * @param string $file_path The path where the PHP file will be saved.
     * @param array $images The list of images to include in the PHP file.
     */
    private function generate_php_file(string $file_path, array $images): void {
        $php_content = '<?php' . PHP_EOL;
        $php_content .= 'echo "<ul>";' . PHP_EOL;
        foreach ($images as $image) {
            $php_content .= 'echo "<li><a href=\'' . esc_url($image['url']) . '\'>' . esc_html($image['url']) . '</a></li>";' . PHP_EOL;
        }
        $php_content .= 'echo "</ul>";' . PHP_EOL;
        $this->write_file($file_path, $php_content);
    }

    /**
     * Writes content to a file.
     *
     * @param string $file_path The path where the file will be saved.
     * @param string $content The content to write to the file.
     */
    private function write_file(string $file_path, string $content): void {
        file_put_contents($file_path, $content);
    }
}

new Mbsm_Image_Sitemap_Generator();