<?php
/**
 * Plugin Name: List Folder Contents with Enhanced SEO
 * Description: Displays folder contents in a single-column list with links, searchable word links, and SEO improvements.
 * Version: 2.0
 * Author: Mbsm.tn
 * Text Domain: list-folder-contents
 */

// Display folder contents
function list_folder_contents_shortcode($atts) {
    $atts = shortcode_atts(['path' => ''], $atts);
    $folder_path = realpath(ABSPATH . $atts['path']);

    if (empty($atts['path']) || !$folder_path || strpos($folder_path, ABSPATH) !== 0 || !is_dir($folder_path)) {
        return __('Invalid or inaccessible folder path.', 'list-folder-contents');
    }

    $contents = array_diff(scandir($folder_path), ['.', '..']);
    $file_types = get_option('lfc_file_types', '');
    if (!empty($file_types)) {
        $file_types_array = array_map('trim', explode(',', $file_types));
        $contents = array_filter($contents, function($item) use ($file_types_array, $folder_path) {
            return in_array(pathinfo($item, PATHINFO_EXTENSION), $file_types_array);
        });
    }

    $per_page = get_option('lfc_files_per_page', 10);
    $total_files = count($contents);
    $total_pages = ceil($total_files / $per_page);
    $current_page = max(1, get_query_var('paged', 1));
    $offset = ($current_page - 1) * $per_page;
    $paged_contents = array_slice($contents, $offset, $per_page);

    $output = '<ul class="list-folder-contents">';
    foreach ($paged_contents as $item) {
        $item_path = $folder_path . '/' . $item;
        $item_url = site_url($atts['path'] . '/' . rawurlencode($item));
        $file_name_with_extension = basename($item);
        $item_name = pathinfo($item, PATHINFO_FILENAME);

        // Split name into words
        $words = preg_split('/[-_ ]+/', $item_name);
        $search_links = array_map(function($word) {
            $search_url = site_url('?s=' . urlencode($word));
            return '<a href="' . esc_url($search_url) . '" target="_blank" rel="noopener noreferrer" title="' . esc_attr(sprintf(__('Search for %s', 'list-folder-contents'), $word)) . '">' . esc_html($word) . '</a>';
        }, $words);

        // Add SEO and security improvements
        // Add an icon (e.g., Font Awesome)
        $icon = '<span class="file-icon">&#x1F4C1;</span>'; // Example icon (folder emoji)

        $output .= sprintf(
            '<li>
                %s
                <a href="%s" target="_blank" rel="noopener noreferrer" title="%s" alt="%s">%s</a><br>%s
            </li>',
            $icon,  // Add the icon before the link
            esc_url($item_url),
            esc_attr(sprintf(__('Open %s', 'list-folder-contents'), $file_name_with_extension)), // Tooltip text
            esc_attr($file_name_with_extension), // ALT text
            esc_html($file_name_with_extension), // Visible link text
            implode(', ', $search_links) // Links for split words
        );
    }
    $output .= '</ul>';

    if ($total_pages > 1) {
        $output .= '<div class="pagination">';
        $output .= paginate_links([
            'total' => $total_pages,
            'current' => $current_page,
            'format' => '?paged=%#%',
            'prev_text' => __('&laquo; Previous'),
            'next_text' => __('Next &raquo;'),
        ]);
        $output .= '</div>';
    }

    return $output;
}
add_shortcode('list_folder_contents', 'list_folder_contents_shortcode');

// Improve speed and styling
function list_folder_contents_enqueue_styles() {
    echo '<style>
        .list-folder-contents {
            list-style-type: none;
            padding: 0;
        }
        .list-folder-contents li {
            margin: 15px 0;
            font-size: 16px;
        }
        .list-folder-contents a {
            color: #0073aa;
            text-decoration: none;
        }
        .list-folder-contents a:hover {
            text-decoration: underline;
        }
        .pagination {
            margin-top: 20px;
        }
        .pagination a, .pagination span {
            margin-right: 5px;
            padding: 5px 10px;
            border: 1px solid #ddd;
            text-decoration: none;
            color: #0073aa;
        }
        .pagination span {
            background-color: #0073aa;
            color: white;
        }
        .file-icon {
            margin-right: 10px;
            font-size: 20px;
        }
    </style>';
}
add_action('wp_head', 'list_folder_contents_enqueue_styles');

// Add settings page to WordPress admin
function lfc_add_settings_page() {
    add_options_page(
        __('List Folder Contents Settings BY MBSMGROUP', 'list-folder-contents'), // Updated title
        __('List Folder Contents BY MBSMGROUP', 'list-folder-contents'), // Updated menu title
        'manage_options',
        'list-folder-contents',
        'lfc_render_settings_page'
    );
}
add_action('admin_menu', 'lfc_add_settings_page');

// Render settings page
function lfc_render_settings_page() {
    ?>
    <div class="wrap">
        <h1><?php _e('List Folder Contents Settings BY MBSMGROUP', 'list-folder-contents'); ?></h1>
        
        <!-- Show Installation Instructions and Features -->
        <h2><?php _e('Installation and Features', 'list-folder-contents'); ?></h2>
        <p><strong><?php _e('Installation:', 'list-folder-contents'); ?></strong></p>
        <ol>
            <li><?php _e('Upload the plugin to your WordPress site under the Plugins section.', 'list-folder-contents'); ?></li>
            <li><?php _e('Activate the plugin.', 'list-folder-contents'); ?></li>
            <li><?php _e('Use the shortcode [list_folder_contents path="your/folder/path"] to display the folder contents on a page.', 'list-folder-contents'); ?></li>
            <li><?php _e('Go to Settings > List Folder Contents to configure the file types and files per page.', 'list-folder-contents'); ?></li>
        </ol>
        
        <p><strong><?php _e('Features:', 'list-folder-contents'); ?></strong></p>
        <ul>
            <li><?php _e('Displays folder contents with configurable file types.', 'list-folder-contents'); ?></li>
            <li><?php _e('SEO enhancement with word links for better search engine interaction.', 'list-folder-contents'); ?></li>
            <li><?php _e('Pagination for large folders to split content into multiple pages.', 'list-folder-contents'); ?></li>
            <li><?php _e('Secure file path handling to prevent unauthorized access.', 'list-folder-contents'); ?></li>
            <li><?php _e('Customizable through the admin settings page.', 'list-folder-contents'); ?></li>
        </ul>
        
        <h3><?php _e('For Developers:', 'list-folder-contents'); ?></h3>
        <p><?php _e('This plugin can be extended by modifying the core functions or adding more filters. To modify the list behavior, look at the list_folder_contents_shortcode function. You can add additional customization by editing the output HTML.', 'list-folder-contents'); ?></p>
        
        <form method="post" action="options.php">
            <?php
            settings_fields('lfc_settings_group');
            do_settings_sections('list-folder-contents');
            ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><?php _e('File Types', 'list-folder-contents'); ?></th>
                    <td>
                        <input type="text" name="lfc_file_types" value="<?php echo esc_attr(get_option('lfc_file_types', '')); ?>" class="regular-text" />
                        <p class="description"><?php _e('Enter allowed file extensions (comma separated, e.g., jpg,png,pdf)', 'list-folder-contents'); ?></p>
                    </td>
                </tr>

                <tr valign="top">
                    <th scope="row"><?php _e('Files Per Page', 'list-folder-contents'); ?></th>
                    <td>
                        <input type="number" name="lfc_files_per_page" value="<?php echo esc_attr(get_option('lfc_files_per_page', 10)); ?>" class="small-text" />
                        <p class="description"><?php _e('Enter the number of files to display per page', 'list-folder-contents'); ?></p>
                    </td>
                </tr>
            </table>

            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// Register settings
function lfc_register_settings() {
    register_setting('lfc_settings_group', 'lfc_file_types');
    register_setting('lfc_settings_group', 'lfc_files_per_page');
}
add_action('admin_init', 'lfc_register_settings');
?>
