<?php
/*
Plugin Name: Mbsm Sitemap Generator
Plugin URI: https://mbsm.tn/
Description: Generates an XML sitemap and additional files (TXT, HTML, ZIP, PHP) for the latest 2000 files in a specific folder.
Version: 1.0
Author: Mbsmgroup
Author URI: https://mbsmgroup.tn
License: GPLv3
*/

class Mbsm_Sitemap_Generator {

    private string $directory = '/wp-content/uploads/Mbsm/';
    private string $output_file = 'sitemap.xml';
    private array $filetypes = [ 'pdf', 'jpg', 'png', 'mp4', 'docx', 'txt', 'zip', 'html', 'xml' ];

    public function __construct() {
        add_action('admin_menu', [$this, 'add_admin_menu']);
    }

    /**
     * Adds an admin menu page.
     */
    public function add_admin_menu() {
        add_menu_page(
            'MBSM Sitemap',  // Page title
            'MBSM Sitemap',  // Menu title
            'manage_options', // Capability
            'mbsm-sitemap',   // Menu slug
            [$this, 'render_admin_page'], // Callback function
            'dashicons-admin-site-alt3', // Icon
            20  // Position
        );
    }

    /**
     * Renders the admin page.
     */
    public function render_admin_page() {
        // Check if the form is submitted
        if (isset($_POST['generate_sitemap'])) {
            $this->generate_sitemap();
            echo '<div class="notice notice-success"><p>Sitemap and files generated successfully!</p></div>';
        }

        $upload_dir = ABSPATH . $this->directory;
        $sitemap_url = home_url($this->directory . $this->output_file);
        $sitemap_path = $upload_dir . $this->output_file;

        $files = [
            'xml' => $sitemap_path,
            'txt' => $upload_dir . 'sitemap.txt',
            'html' => $upload_dir . 'sitemap.html',
            'zip' => $upload_dir . 'sitemap.zip',
            'php' => $upload_dir . 'sitemap.php',
        ];
        ?>
        <div class="wrap">
            <h1>MBSM Sitemap Generator</h1>
            <p>Click the button below to generate the sitemap and additional files for the latest 2000 files in <code><?php echo esc_html($this->directory); ?></code>.</p>
            <form method="post" action="">
                <input type="hidden" name="generate_sitemap" value="1">
                <?php submit_button('Generate Sitemap and Files'); ?>
            </form>

            <?php if (file_exists($sitemap_path)) : ?>
                <h2>Generated Files</h2>
                <table class="wp-list-table widefat fixed striped">
                    <thead>
                        <tr>
                            <th>File Name</th>
                            <th>File URL</th>
                            <th>File Size</th>
                            <th>Last Modified</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php foreach ($files as $type => $path) : ?>
                            <?php if (file_exists($path)) : ?>
                                <tr>
                                    <td><?php echo esc_html(basename($path)); ?></td>
                                    <td>
                                        <a href="<?php echo esc_url(home_url($this->directory . basename($path))); ?>" target="_blank">
                                            <?php echo esc_html(basename($path)); ?>
                                        </a>
                                    </td>
                                    <td><?php echo esc_html(size_format(filesize($path))); ?></td>
                                    <td><?php echo esc_html(date('F d, Y H:i:s', filemtime($path))); ?></td>
                                </tr>
                            <?php endif; ?>
                        <?php endforeach; ?>
                    </tbody>
                </table>
            <?php else : ?>
                <p>No sitemap found. Please generate the sitemap first.</p>
            <?php endif; ?>
        </div>
        <?php
    }

    /**
     * Scans the directory and generates the sitemap and additional files.
     */
    public function generate_sitemap() {
        $upload_dir = ABSPATH . $this->directory;

        // ????? ?????? ??? ?? ??? ???????
        if (!file_exists($upload_dir)) {
            wp_mkdir_p($upload_dir);
        }

        // ??? ??????? ??????? ???? ?? ??????? ?????? ???????? ??????? ???
        $this->delete_generated_files($upload_dir);

        // ??? ???? 2000 ???
        $files = $this->scan_directory($upload_dir);

        // Generate XML sitemap
        $xml_content = $this->generate_xml_content($files);
        $this->write_file($upload_dir . $this->output_file, $xml_content);

        // Generate additional files
        $this->generate_txt_file($upload_dir . 'sitemap.txt', $files);
        $this->generate_html_file($upload_dir . 'sitemap.html', $files);
        $this->generate_zip_file($upload_dir . 'sitemap.zip', $files);
        $this->generate_php_file($upload_dir . 'sitemap.php', $files);
    }

    /**
     * Deletes only the files generated by this plugin.
     *
     * @param string $dir The directory path.
     */
    private function delete_generated_files(string $dir): void {
        if (is_dir($dir)) {
            // ????? ??????? ???? ?? ??????? ?????? ???????? ???????
            $generated_files = [
                'sitemap.xml',
                'sitemap.txt',
                'sitemap.html',
                'sitemap.zip',
                'sitemap.php',
            ];

            foreach ($generated_files as $file) {
                $file_path = $dir . $file;
                if (is_file($file_path)) {
                    unlink($file_path); // ??? ?????
                }
            }
        }
    }

    /**
     * Scans a directory and returns the latest 2000 files with specific extensions.
     *
     * @param string $dir The directory path to scan.
     * @return array The list of files with their URLs and modification times.
     */
    private function scan_directory(string $dir): array {
        $files = [];
        if (is_dir($dir)) {
            $all_files = scandir($dir, SCANDIR_SORT_DESCENDING); // ????? ??????? ?? ?????? ??? ??????

            foreach ($all_files as $file) {
                if (in_array($file, ['.', '..'], true)) {
                    continue;
                }

                $file_path = $dir . $file;
                $extension = pathinfo($file, PATHINFO_EXTENSION);

                if (in_array($extension, $this->filetypes, true)) {
                    $mod_time = gmdate('c', filemtime($file_path)); // Get file modification time
                    $files[] = [
                        'url' => home_url($this->directory . $file),
                        'mod' => $mod_time
                    ];

                    // ????? ????? ??? ?????? ??? 2000 ???
                    if (count($files) >= 2000) {
                        break;
                    }
                }
            }
        }
        return $files;
    }

    /**
     * Generates XML content for the sitemap.
     *
     * @param array $files The list of files to include in the sitemap.
     * @return string The XML content.
     */
    private function generate_xml_content(array $files): string {
        $xml = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL;
        $xml .= '<urlset xmlns="https://www.sitemaps.org/schemas/sitemap/0.9">' . PHP_EOL;

        foreach ($files as $file) {
            $xml .= '<url>' . PHP_EOL;
            $xml .= '<loc>' . esc_url($file['url']) . '</loc>' . PHP_EOL;
            $xml .= '<lastmod>' . esc_html($file['mod']) . '</lastmod>' . PHP_EOL;
            $xml .= '</url>' . PHP_EOL;
        }

        $xml .= '</urlset>';
        return $xml;
    }

    /**
     * Generates a TXT file.
     *
     * @param string $file_path The path where the TXT file will be saved.
     * @param array $files The list of files to include in the TXT file.
     */
    private function generate_txt_file(string $file_path, array $files): void {
        $txt_content = '';
        foreach ($files as $file) {
            $txt_content .= $file['url'] . PHP_EOL;
        }
        $this->write_file($file_path, $txt_content);
    }

    /**
     * Generates an HTML file.
     *
     * @param string $file_path The path where the HTML file will be saved.
     * @param array $files The list of files to include in the HTML file.
     */
    private function generate_html_file(string $file_path, array $files): void {
        $html_content = '<!DOCTYPE html><html><head><title>Sitemap</title></head><body><ul>';
        foreach ($files as $file) {
            $html_content .= '<li><a href="' . esc_url($file['url']) . '">' . esc_html($file['url']) . '</a></li>';
        }
        $html_content .= '</ul></body></html>';
        $this->write_file($file_path, $html_content);
    }

    /**
     * Generates a ZIP file.
     *
     * @param string $file_path The path where the ZIP file will be saved.
     * @param array $files The list of files to include in the ZIP file.
     */
    private function generate_zip_file(string $file_path, array $files): void {
        $zip = new ZipArchive();
        if ($zip->open($file_path, ZipArchive::CREATE) === TRUE) {
            foreach ($files as $file) {
                $file_name = basename($file['url']);
                $zip->addFromString($file_name, file_get_contents(ABSPATH . $this->directory . $file_name));
            }
            $zip->close();
        }
    }

    /**
     * Generates a PHP file.
     *
     * @param string $file_path The path where the PHP file will be saved.
     * @param array $files The list of files to include in the PHP file.
     */
    private function generate_php_file(string $file_path, array $files): void {
        $php_content = '<?php' . PHP_EOL;
        $php_content .= 'echo "<ul>";' . PHP_EOL;
        foreach ($files as $file) {
            $php_content .= 'echo "<li><a href=\'' . esc_url($file['url']) . '\'>' . esc_html($file['url']) . '</a></li>";' . PHP_EOL;
        }
        $php_content .= 'echo "</ul>";' . PHP_EOL;
        $this->write_file($file_path, $php_content);
    }

    /**
     * Writes content to a file.
     *
     * @param string $file_path The path where the file will be saved.
     * @param string $content The content to write to the file.
     */
    private function write_file(string $file_path, string $content): void {
        file_put_contents($file_path, $content);
    }
}

new Mbsm_Sitemap_Generator();